/// <reference path="../ts_definitions/ScreenCalcTypes.d.ts" />
/**
* Allows screen information to be easily stored and calculated
*/
declare class ScreenCalc {
    private pixelWidth;
    private pixelHeight;
    private pixelCount;
    private pixelDensity;
    private ratio;
    private physicalWidth;
    private physicalHeight;
    private diagonalSize;
    /** Magic number that results in "expected" ratio for common screen resolutions */
    private static DEFAULT_RATIO_PRECISION;
    constructor(properties?: ScreenConstructor);
    /**
    * Set data for one or more properties.
    * If 'replace' param is set to true, properties that aren't passed will be set to null.
    */
    public setData(properties: ScreenConstructor, replace?: boolean): void;
    /**
    * Attempts to calculate/return the pixel height of the screen.
    * Returns null if there is insufficient data.
    */
    public getPixelHeight(): number;
    /**
    * Attempts to calculate/return the pixel width of the screen.
    * Returns null if there is insufficient data.
    */
    public getPixelWidth(): number;
    /**
    * Attempts to calculate/return the screen's physical height.
    * Returns null if there is insufficient data.
    */
    public getPhysicalHeight(): number;
    /**
    * Attempts to calculate/return the screen's physical width.
    * Returns null if there is insufficient data.
    */
    public getPhysicalWidth(): number;
    /**
    * Attempts to calculate/return the screen's diagonal size.
    * Returns null if there is insufficient data.
    */
    public getDiagonalSize(): number;
    /**
    * Returns the number of pixels per unit in the display
    * (will be ppi if units are in, or ppcm if units are cm)
    */
    public getPixelDensity(): number;
    /** Returns the area of the display in square units */
    public getArea(): number;
    /** Returns the total number of pixels in the screen */
    public getPixelCount(): number;
    /**
    * Returns the screen's width divided by its height.
    * For example, the ratio of a 1920x1080 display would
    * be 1.78 when rounded to two decimal places.
    */
    public getRatio(): number;
    /**
    * Returns an object with 'width' and 'height' properties containing a simplified ratio for the display.
    * Additionally, a 'difference' property is included which contains the difference between the simplified
    * and original ratio (0 if the simplified ratio is exact). For example, a 1366x768 display would (by default)
    * return { width: 16, height: 9, difference: -0.0008680555555555802 }.
    * Returns null if there is not enough data to calculate the ratio.
    * @param precision (optional) pass a number between -1 and 0 for precision (defaults to 5.0e-3).
    * The closer the number is to zero the greater the precision. For example, if 1.0e-6 is passed,
    * the return value for a 1366x768 display would be { width: 683, height: 384, difference: 0 }.
    */
    public getSimpleRatio(precision?: number): {
        width: number;
        height: number;
        difference: number;
    };
    /**
    * Returns the same simplified ratio as getSimpleRatio(), but as a string in the format width:height.
    * For example, a 1920x1080 display would return "16:9". If the ratio is imprecise, a tilde (~) character
    * is prepended to the string (i.e. "~16x9").
    * Returns null if there is not enough data to calculate the ratio.
    * @param precision (optional) pass a number between -1 and 0 for precision (defaults to 5.0e-3).
    * The closer the number is to zero the greater the precision. For example, if 1.0e-6 is passed,
    * the return value for a 1366x768 display would be the precise ratio "683:384".
    */
    public getStringRatio(precision?: number): string;
    /**
    * Tries to calculate width and height in the same units (used for ratio calculation).
    * Returns an object with height and width properties, or null if insufficient data.
    */
    private calculateWidthAndHeight();
    private setPixelWidth(pixelWidth);
    private setPixelHeight(pixelHeight);
    private setPixelCount(pixels);
    private setPixelDensity(density);
    private setRatio(ratio);
    private setPhysicalWidth(width);
    private setPhysicalHeight(height);
    private setDiagonalSize(diagonalSize);
}
export = ScreenCalc;
